package com.simuwang.base.common.util;

import com.simuwang.base.common.conts.DateConst;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class DateUtils {
    public static final Map<Integer, String> charMap = new HashMap();

    public static final Pattern p = Pattern.compile("^(\\d+)\\D*(\\d*)\\D*(\\d*)\\D*(\\d*)\\D*(\\d*)\\D*(\\d*)");

    static {

        charMap.put(1, "y");

        charMap.put(2, "M");

        charMap.put(3, "d");

        charMap.put(4, "H");

        charMap.put(5, "m");

        charMap.put(6, "s");

    }

    private static final Logger logger = LoggerFactory.getLogger(DateUtils.class);

    /**
     * 获得当前日期 yyyy-MM-dd HH:mm:ss
     *
     * @return 2019-08-27 14:12:40
     */
    public static String getCurrentTime() {
        // 小写的hh取得12小时，大写的HH取的是24小时
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = new Date();
        return df.format(date);
    }

    /**
     * 获取系统当前时间戳
     *
     * @return 1566889186583
     */
    public static String getSystemTime() {
        String current = String.valueOf(System.currentTimeMillis());
        return current;
    }


    /**
     * 获取当前日期 yyyy-MM-dd
     *
     * @return 2019-08-27
     */
    public static String getTodayDate() {
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        return sdf.format(date);
    }


    public static String replaceEachChar(String s, String c) {
        StringBuilder sb = new StringBuilder("");

        for (Character c1 : s.toCharArray()) {
            if (c1 != ' ') {
                sb.append(c);
            }
        }
        return sb.toString();
    }

    /**
     * @param dateString
     * @return
     */
    public static Date StringToDate(String dateString) {
        dateString.trim().replaceAll("[a-zA-Z]", " ");
        //支持13位时间戳
        if (Pattern.matches("^[-+]?\\d{13}$", dateString)) {
            return new Date(Long.parseLong(dateString));
        }
        Matcher m = p.matcher(dateString);
        StringBuilder sb = new StringBuilder(dateString);
        //从被匹配的字符串中，充index = 0的下表开始查找能够匹配pattern的子字符串。m.matches()的意思是尝试将整个区域与模式匹配，不一样。
        if (m.find(0)) {
            int count = m.groupCount();
            for (int i = 1; i <= count; i++) {
                for (Map.Entry<Integer, String> entry : charMap.entrySet()) {
                    if (entry.getKey() == i) {
                        sb.replace(m.start(i), m.end(i), replaceEachChar(m.group(i), entry.getValue()));
                    }
                }
            }
        } else {
            logger.error("错误的日期格式" + dateString);
        }
//        System.out.println(sb);
        Date date = null;
        try {
            date = new SimpleDateFormat(sb.toString()).parse(dateString);
        } catch (ParseException e) {
        }
        return date;
    }

    /**
     * @param strDate 字符串类型的时间
     * @param pattern 格式
     * @return 日期
     */
    public static Date StringToDate(String strDate, String pattern) {
        Date date = null;
        try {
            date = new SimpleDateFormat(pattern).parse(strDate);
        } catch (ParseException e) {
            logger.error("时间转换报错", e.getMessage(), e);
        }
        return date;
    }

    /**
     * @param date    时间
     * @param pattern 格式
     * @return 日期
     */
    public static String DateToString(Date date, String pattern) {
        DateFormat sdf = new SimpleDateFormat(pattern);
        String strDate = "";
        try {
            strDate = sdf.format(date);
        } catch (Exception e) {
            logger.error("时间转换报错", e.getMessage(), e);
        }
        return strDate;
    }

    /**
     * 得到两个时间差  格式yyyy-MM-dd HH:mm:ss
     *
     * @param start 2019-06-27 14:12:40
     * @param end   2019-08-27 14:12:40
     * @return 5270400000
     */
    public static long dateSubtraction(String start, String end) {
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            Date date1 = df.parse(start);
            Date date2 = df.parse(end);
            return date2.getTime() - date1.getTime();
        } catch (ParseException e) {
            e.printStackTrace();
            return 0;
        }
    }

    /**
     * 得到两个时间差
     *
     * @param start 开始时间
     * @param end   结束时间
     * @return
     */
    public static long dateTogether(Date start, Date end) {
        return end.getTime() - start.getTime();
    }

    /**
     * 转化long值的日期为yyyy-MM-dd  HH:mm:ss.SSS格式的日期
     *
     * @param millSec 日期long值  5270400000
     * @return 日期，以yyyy-MM-dd  HH:mm:ss.SSS格式输出 1970-03-03  08:00:00.000
     */
    public static String transferLongToDate(String millSec) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd  HH:mm:ss.SSS");
        Date date = new Date(Long.parseLong(millSec));
        return sdf.format(date);
    }

    /**
     * 获得当前日期 yyyy-MM-dd HH:mm:ss
     *
     * @return
     */
    public static String getOkDate(String date) {
        try {
            if (StringUtils.isEmpty(date)) {
                return null;
            }
            Date date1 = new SimpleDateFormat("EEE MMM dd HH:mm:ss Z yyyy", Locale.ENGLISH).parse(date);
            //格式化
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            return sdf.format(date1);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     * 获取当前日期是一个星期的第几天
     *
     * @return 2
     */
    public static int getDayOfWeek() {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());
        return cal.get(Calendar.DAY_OF_WEEK) - 1;
    }

    /**
     * 获取当前日期是一个月的第几周
     *
     * @return 2
     */
    public static int getWeekofMonth() {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());
        return cal.get(Calendar.WEEK_OF_MONTH);
    }


    /**
     * 判断当前时间是否在[startTime, endTime]区间，注意时间格式要一致
     *
     * @param nowTime     当前时间
     * @param dateSection 时间区间   2018-01-08,2019-09-09
     * @return
     * @author jqlin
     */
    public static boolean isEffectiveDate(Date nowTime, String dateSection) {
        try {
            String[] times = dateSection.split(",");
            String format = "yyyy-MM-dd";
            Date startTime = new SimpleDateFormat(format).parse(times[0]);
            Date endTime = new SimpleDateFormat(format).parse(times[1]);
            if (nowTime.getTime() == startTime.getTime()
                    || nowTime.getTime() == endTime.getTime()) {
                return true;
            }
            Calendar date = Calendar.getInstance();
            date.setTime(nowTime);

            Calendar begin = Calendar.getInstance();
            begin.setTime(startTime);

            Calendar end = Calendar.getInstance();
            end.setTime(endTime);

            if (isSameDay(date, begin) || isSameDay(date, end)) {
                return true;
            }
            if (date.after(begin) && date.before(end)) {
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public static boolean isSameDay(Calendar cal1, Calendar cal2) {
        if (cal1 != null && cal2 != null) {
            return cal1.get(0) == cal2.get(0) && cal1.get(1) == cal2.get(1) && cal1.get(6) == cal2.get(6);
        } else {
            throw new IllegalArgumentException("The date must not be null");
        }
    }

    public static long getTimeByDate(String time) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            Date date = format.parse(time);
            //日期转时间戳（毫秒）
            return date.getTime();
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }
    }

    /**
     * 获取当前小时 ：2019-08-23 17
     *
     * @return 2019-08-27 17
     */
    public static String getCurrentHour() {
        GregorianCalendar calendar = new GregorianCalendar();
        int hour = calendar.get(Calendar.HOUR_OF_DAY);
        if (hour < 10) {
            return DateUtils.getCurrentTime() + " 0" + hour;
        }
        return DateUtils.getTodayDate() + " " + hour;
    }

    /**
     * 获取当前时间一个小时前
     *
     * @return 2019-08-27 16
     */
    public static String getCurrentHourBefore() {
        GregorianCalendar calendar = new GregorianCalendar();
        int hour = calendar.get(Calendar.HOUR_OF_DAY);
        if (hour > 0) {
            hour = calendar.get(Calendar.HOUR_OF_DAY) - 1;
            if (hour < 10) {
                return DateUtils.getTodayDate() + " 0" + hour;
            }
            return DateUtils.getTodayDate() + " " + hour;
        }
        //获取当前日期前一天
        return DateUtils.getBeforeDay() + " " + 23;
    }

    /**
     * 获取当前日期前一天
     *
     * @return 2019-08-26
     */
    public static String getBeforeDay() {

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        date = calendar.getTime();
        return sdf.format(date);
    }

    /**
     * 获取当前日期前一天
     *
     * @return 2019-08-26
     */
    public static String getBeforeDayTime() {

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        date = calendar.getTime();
        return sdf.format(date);
    }

    /**
     * 获取当前日期前几天
     *
     * @param num 前几天的天数
     * @return
     */
    public static String getBeforeNumDayTime(Integer num) {

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH, 0 - num);
        date = calendar.getTime();
        return sdf.format(date);
    }


    /**
     * 获取最近七天
     *
     * @return 2019-08-20
     */
    public static String getServen() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        Calendar c = Calendar.getInstance();

        c.add(Calendar.DATE, -7);

        Date monday = c.getTime();

        String preMonday = sdf.format(monday);

        return preMonday;
    }

    /**
     * 获取最近一个月
     *
     * @return 2019-07-27
     */
    public static String getOneMonth() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        Calendar c = Calendar.getInstance();

        c.add(Calendar.MONTH, -1);

        Date monday = c.getTime();

        String preMonday = sdf.format(monday);

        return preMonday;
    }

    /**
     * 获取最近三个月
     *
     * @return 2019-05-27
     */
    public static String getThreeMonth() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        Calendar c = Calendar.getInstance();

        c.add(Calendar.MONTH, -3);

        Date monday = c.getTime();

        String preMonday = sdf.format(monday);

        return preMonday;
    }

    /**
     * 获取最近一年
     *
     * @return 2018-08-27
     */
    public static String getOneYear() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Calendar c = Calendar.getInstance();
        c.add(Calendar.YEAR, -1);
        Date start = c.getTime();
        String startDay = sdf.format(start);
        return startDay;
    }


    private static int month = Calendar.getInstance().get(Calendar.MONTH) + 1;

    /**
     * 获取今年月份数据
     * 说明 有的需求前端需要根据月份查询每月数据，此时后台给前端返回今年共有多少月份
     *
     * @return [1, 2, 3, 4, 5, 6, 7, 8]
     */
    public static List getMonthList() {
        List list = new ArrayList();
        for (int i = 1; i <= month; i++) {
            list.add(i);
        }
        return list;
    }

    /**
     * 返回当前年度季度list
     * 本年度截止目前共三个季度，然后根据1,2,3分别查询相关起止时间
     *
     * @return [1, 2, 3]
     */
    public static List getQuartList() {
        int quart = month / 3 + 1;
        List list = new ArrayList();
        for (int i = 1; i <= quart; i++) {
            list.add(i);
        }
        return list;
    }

    /**
     * 根据yyyyMM的格式，返回yyyyQq的格式，如输入2021-12，返回2021Q4
     *
     * @param yearAndMonth yyyy-MM格式
     * @return
     */
    public static String getQuartByYearAndMonth(String yearAndMonth) {
        DateTimeFormatter QUARTER_FORMAT = DateTimeFormatter.ofPattern("yyyy-'Q'q");
        DateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM");
        try {
            return DATE_FORMAT.parse(yearAndMonth).toInstant().atZone(ZoneId.systemDefault()).format(QUARTER_FORMAT);
        } catch (Exception e) {
            logger.error("getQuartByYearAndMonth format Exception: {}", e.getMessage(), e);
            return null;
        }
    }

    /**
     * 对日期操作，增加N个月，如果为负数则为减去N个月
     *
     * @param date   格式yyyy-MM
     * @param amount 增加的月份，负数代表减去
     * @return 返回和原格式一样的String值
     */
    public static String addMonth(String date, int amount) {
        if (amount == 0) {
            return date;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
        try {
            Date initDate = sdf.parse(date);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(initDate);
            calendar.add(Calendar.MONTH, amount);
            return sdf.format(calendar.getTime());
        } catch (Exception e) {
            logger.error("addMonth format Exception: {}", e.getMessage(), e);
            return null;
        }
    }

    /**
     * 对日期操作，增加N个月，如果为负数则为减去N个月
     *
     * @param date    日期
     * @param amount  增加的月份，负数代表减去
     * @param pattern 日期格式
     * @return 返回和原格式一样的String值
     */
    public static String addMonth(String date, int amount, String pattern) {
        if (amount == 0) {
            return date;
        }
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        try {
            Date initDate = sdf.parse(date);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(initDate);
            calendar.add(Calendar.MONTH, amount);
            return sdf.format(calendar.getTime());
        } catch (Exception e) {
            logger.error("addMonth format Exception: {}", e.getMessage(), e);
            return null;
        }
    }

    public static Date addMonth(Date date, int amount) {
        if (amount == 0) {
            return date;
        }

        Calendar cld = Calendar.getInstance();
        cld.setTime(date);
        cld.add(Calendar.MONTH, amount);
        return cld.getTime();
    }

    /**
     * 把日期格式变一下
     *
     * @param dateStr    字符串类型的日期
     * @param oldPattern 原格式
     * @param newPattern 新格式
     * @return
     * @throws ParseException
     */
    public static String format(String dateStr, String oldPattern, String newPattern) {
        if (StringUtils.isEmpty(dateStr) || StringUtils.isEmpty(oldPattern) || StringUtils.isEmpty(newPattern)) {
            return null;
        }
        try {
            SimpleDateFormat sdf = new SimpleDateFormat(oldPattern);
            Date date = sdf.parse(dateStr);
            sdf = new SimpleDateFormat(newPattern);
            return sdf.format(date);
        } catch (Exception e) {
            logger.error("format method Exception: {}", e.getMessage(), e);
            return null;
        }
    }

    /**
     * 将日期格式转化为yyyyWW的格式
     * 输出格式为年周，遇到周跨年的场景遵照ISO8601标准
     * 所以可能会出现将2001-12-31转化后得到200201的场景
     *
     * @return
     */


    public static String getMonthByYearWeek(int year, int week) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(year, 0, 1);
        int weeks = 0;
        while (weeks <= week) {
            calendar.add(Calendar.WEEK_OF_YEAR, 1);
            weeks++;
        }
        calendar.add(Calendar.MONTH, -1);
        Date date = calendar.getTime();
        SimpleDateFormat sfd = new SimpleDateFormat("yyyy-MM");
        String yearMonth = sfd.format(date);
        return yearMonth;
    }

    public static String getTheSpecifiedYearAndWeekDate(String date) {
        date = date.replace("-", "");
        Integer year = Integer.valueOf(date.substring(0, 4));
        Integer week = Integer.valueOf(date.substring(4, 6));
        Map<String, String> timeMap = new HashMap<>();
        Calendar calendar = Calendar.getInstance();
        calendar.set(year, 0, 1);
        int weekDay = calendar.get(Calendar.DAY_OF_WEEK);
        //计算当年第一周还剩几天
        int dayOfWeek = 0;
        if (1 == weekDay) {
            dayOfWeek = 1;
        }
        if (2 == weekDay) {
            dayOfWeek = 7;
        }
        if (3 == weekDay) {
            dayOfWeek = 6;
        }
        if (4 == weekDay) {
            dayOfWeek = 5;
        }
        if (5 == weekDay) {
            dayOfWeek = 4;
        }
        if (6 == weekDay) {
            dayOfWeek = 3;
        }
        if (7 == weekDay) {
            dayOfWeek = 2;
        }
        if (dayOfWeek > 3) {
            week = week - 2;
        } else {
            //周数减去第一周再减去要得到的周
            week = week - 1;
        }
        calendar.add(Calendar.DAY_OF_YEAR, week * 7 + dayOfWeek);
        timeMap.put("fisrstDay", new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime()));
        calendar.add(Calendar.DAY_OF_YEAR, 4);
        timeMap.put("friday", new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime()));
        return new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime());
    }

    public static String format(Date date, String pattern) {
        if (date == null) {
            return "";
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat(pattern);
        return dateFormat.format(date);
    }

    public static Date parse(String dateStr, String pattern) {
        SimpleDateFormat dateFormat = new SimpleDateFormat(pattern);
        try {
            return dateFormat.parse(dateStr);
        } catch (ParseException e) {
            throw new RuntimeException("Parse date error: " + dateStr);
        }
    }



    public static Calendar getCalendarByDate(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar;
    }

    /**
     * 判断是否在今天的6点到19点30之间
     *
     * @param nowTime
     * @return
     */
    public static boolean isWorkingTime(Date nowTime) {

        Calendar date = Calendar.getInstance();
        date.setTime(nowTime);

        Calendar begin = Calendar.getInstance();
        begin.setTime(new Date());
        begin.set(Calendar.HOUR_OF_DAY, 6);
        begin.set(Calendar.MINUTE, 0);
        begin.set(Calendar.SECOND, 0);

        Calendar end = Calendar.getInstance();
        end.setTime(new Date());
        end.set(Calendar.HOUR_OF_DAY, 19);
        end.set(Calendar.MINUTE, 30);
        end.set(Calendar.SECOND, 0);

        if (date.after(begin) && date.before(end)) {
            return true;
        } else {
            return false;
        }
    }

    public static Boolean isBefore(String beginTime, String endTime) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date bt = null;
        Date et = null;
        try {
            bt = sdf.parse(beginTime);
            et = sdf.parse(endTime);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        if (bt.before(et)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @param date
     * @param startDate
     * @param endDate
     * @return (startDate, endDate]
     */
    public static Boolean isBetweenOrEquelsModel1(Date date, Date startDate, Date endDate) {
        if (date == null || startDate == null || endDate == null) {
            return false;
        }

        if ((date.before(endDate) || date.equals(endDate)) && (date.after(startDate))) {
            return true;
        }

        return false;
    }

    /**
     * @param date
     * @param startDate
     * @param endDate
     * @return [startDate, endDate]
     */
    public static Boolean isBetweenOrEquelsModel2(Date date, Date startDate, Date endDate) {
        if (date == null || startDate == null || endDate == null) {
            return false;
        }

        if ((date.before(endDate) || date.equals(endDate)) && (date.after(startDate) || date.equals(startDate))) {
            return true;
        }

        return false;
    }


    public static Boolean isBetweenAfterModel2(Date date, Date startDate, Date endDate) {
        if (date == null || startDate == null || endDate == null) {
            return false;
        }

        if ((date.before(endDate) || date.equals(endDate)) && (date.after(startDate))) {
            return true;
        }

        return false;
    }


    public static Boolean isBeforeOrEquels(Date date, Date endDate) {
        if (date == null || endDate == null) {
            return false;
        }

        if (date.before(endDate) || date.equals(endDate)) {
            return true;
        }

        return false;
    }

    public static Date getMonthLastDate(Date date) {
        Calendar cld = Calendar.getInstance();
        cld.setTime(date);
        cld.add(Calendar.MONTH, 1);
        cld.set(Calendar.DAY_OF_MONTH, 0);
        return cld.getTime();
    }

    public static Date getMonthFirstDate(Date date) {
        Calendar cld = Calendar.getInstance();
        cld.setTime(date);
        cld.set(Calendar.DAY_OF_MONTH, 1);
        cld.set(Calendar.HOUR_OF_DAY, 0);
        cld.set(Calendar.MINUTE, 0);
        cld.set(Calendar.SECOND, 0);
        cld.set(Calendar.MILLISECOND, 0);
        return cld.getTime();
    }



    /**
     * 获取日期是当月的第几天
     *
     * @return 2
     */
    public static int getDayOfMonth(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        return cal.get(Calendar.DAY_OF_MONTH);
    }

    /**
     * 对日期操作，增加N天，如果为负数则为减去N天
     *
     * @param date    日期
     * @param amount  增加的月份，负数代表减去
     * @param pattern 日期格式
     * @return 返回Date
     */
    public static Date addDay(String date, int amount, String pattern) {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        try {
            Date initDate = sdf.parse(date);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(initDate);
            calendar.add(Calendar.DAY_OF_YEAR, amount);
            return calendar.getTime();
        } catch (Exception e) {
            logger.error("addMonth format Exception: {}", e.getMessage(), e);
            return null;
        }
    }

    /**
     *
     */
    public static String getDayOfYearMonth(int year, int month, int day) {
        Calendar cal = Calendar.getInstance();
        //设置年份
        cal.set(Calendar.YEAR, year);
        //设置月份
        cal.set(Calendar.MONTH, month - 1);
        //设置日历中月份的最小天数
        cal.set(Calendar.DAY_OF_MONTH, day);
        //格式化日期
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        return sdf.format(cal.getTime());
    }

    /**
     * 获取某月的最后一天
     */
    public static String getLastDayOfMonth(int year, int month) {
        Calendar cal = Calendar.getInstance();
        //设置年份
        cal.set(Calendar.YEAR, year);
        //设置月份
        cal.set(Calendar.MONTH, month - 1);
        //获取某月最大天数
        int lastDay = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
        //设置日历中月份的最大天数
        cal.set(Calendar.DAY_OF_MONTH, lastDay);
        //格式化日期
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        return sdf.format(cal.getTime());
    }

    /**
     * 获取上周末
     *
     * @return
     */
    public static String getLastWeekEndDate() {
        Calendar cal = Calendar.getInstance();
        int n = cal.get(Calendar.DAY_OF_WEEK) - 1;
        if (n == 0) {
            n = 7;
        }
        cal.add(Calendar.DATE, -n);

        Date time = cal.getTime();
        return DateToString(time, DateConst.YYYY_MM_DD);
    }

    /**
     * 获取上月末
     *
     * @return
     */
    public static String getLastMonthEndDate() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.DAY_OF_MONTH, 1);
        cal.add(Calendar.DATE, -1);

        Date time = cal.getTime();
        return DateToString(time, DateConst.YYYY_MM_DD);
    }

    /**
     * 获取上月末
     *
     * @return
     */
    public static String getLastMonthEndDate(String dateStr) {
        Date date = StringToDate(dateStr, DateConst.YYYY_MM_DD);
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(Calendar.DAY_OF_MONTH, 1);
        cal.add(Calendar.DATE, -1);

        Date time = cal.getTime();
        return DateToString(time, DateConst.YYYY_MM_DD);
    }

    /**
     * 获取月份 + 1
     *
     * @return
     */
    public static String getLaterMonth(String dateStr) {
        Date date = StringToDate(dateStr, DateConst.YYYY_MM_DD);
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.add(Calendar.MONTH, 1);

        Date time = cal.getTime();
        return DateToString(time, DateConst.YYYY_MM_DD);
    }

    /**
     * 获取月份 - 1
     *
     * @return
     */
    public static String getBeforeMonth(String dateStr) {
        Date date = StringToDate(dateStr, DateConst.YYYY_MM_DD);
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.add(Calendar.MONTH, -1);

        Date time = cal.getTime();
        return DateToString(time, DateConst.YYYY_MM_DD);
    }

    /**
     * 获取本月初
     *
     * @return
     */
    public static String getThisMonthFirstDate(String dateStr) {
        Date date = StringToDate(dateStr, DateConst.YYYY_MM_DD);
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(Calendar.DAY_OF_MONTH, 1);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);

        Date time = cal.getTime();
        return DateToString(time, DateConst.YYYY_MM_DD);
    }

    /**
     * 获取本月第n天
     *
     * @return
     */
    public static String getThisMonthNumDate(String dateStr, Integer n) {
        Date date = StringToDate(dateStr, DateConst.YYYY_MM_DD);
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(Calendar.DAY_OF_MONTH, n);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);

        Date time = cal.getTime();
        return DateToString(time, DateConst.YYYY_MM_DD);
    }

    /**
     * 获取本月末
     *
     * @return
     */
    public static String getThisMonthEndDate(String dateStr) {
        Date date = StringToDate(dateStr, DateConst.YYYY_MM_DD);
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));

        Date time = cal.getTime();
        return DateToString(time, DateConst.YYYY_MM_DD);
    }


    /**
     * 上年末
     *
     * @return
     */
    public static String getLastDayOfLastYear() {

        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);

        Calendar calendar2 = Calendar.getInstance();
        calendar2.clear();
        calendar2.set(Calendar.YEAR, year - 1);
        calendar2.roll(Calendar.DAY_OF_YEAR, -1);
        Date time = calendar2.getTime();
        return DateToString(time, DateConst.YYYY_MM_DD);
    }

    public static String getLastDayOfLastYear(String dateStr) {

        Date date = StringToDate(dateStr, DateConst.YYYY_MM_DD);

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int year = calendar.get(Calendar.YEAR);

        Calendar calendar2 = Calendar.getInstance();
        calendar2.clear();
        calendar2.set(Calendar.YEAR, year - 1);
        calendar2.roll(Calendar.DAY_OF_YEAR, -1);
        Date time = calendar2.getTime();
        return DateToString(time, DateConst.YYYY_MM_DD);
    }

    public static String getThisYearFirstDay() {
        Date time = new Date();
        try {
            Calendar currCal = Calendar.getInstance();
            Calendar calendar = Calendar.getInstance();
            calendar.clear();
            calendar.set(Calendar.YEAR, currCal.get(Calendar.YEAR));
            time = calendar.getTime();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return DateToString(time, DateConst.YYYY_MM_DD);
    }
    public static int getQuarterOfYear(Date date) {

        Calendar calendar = Calendar.getInstance();

        calendar.setTime(date);

        return calendar.get(Calendar.MONTH) / 3 + 1;

    }

    public static String getPreDate(String dateStr) {
        Date date = StringToDate(dateStr, DateConst.YYYY_MM_DD);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, -1);
        Date preDate = calendar.getTime();
        String preDateStr = DateToString(preDate, DateConst.YYYY_MM_DD);
        return preDateStr;
    }


    /**
     * 2021-12   2022-11
     *
     * @param pastDate
     * @return
     */
    public static int getDiffMonthFromNow(String pastDate) {
        java.time.YearMonth beginDate = java.time.YearMonth.parse(pastDate, DateTimeFormatter.ofPattern(DateConst.YYYY_MM_DD));
        LocalDate now = LocalDate.now();
        int diffMonth = (now.getYear() - beginDate.getYear()) * 12 + (now.getMonthValue() - beginDate.getMonthValue());
        return diffMonth;
    }

    /**
     * 2021-12   2022-11
     * 根据type来获取两个时间间隔的天数/周数/月份
     *
     * @param pastDate
     * @return
     */
    public static long getDiffByType(String pastDate, String newDate, String type) {
        LocalDate past = LocalDate.parse(pastDate, DateTimeFormatter.ofPattern(DateConst.YYYY_MM_DD));
        LocalDate current = LocalDate.parse(newDate, DateTimeFormatter.ofPattern(DateConst.YYYY_MM_DD));
        if ("Day".equals(type)) {
            return current.toEpochDay() - past.toEpochDay();
        } else if ("Month".equals(type)) {
            return (current.toEpochDay() - past.toEpochDay()) / 30;
        }

        return 0;
    }

    /**
     * @param smallDate
     * @param bigDate
     * @desc 获取两个日期之间的天数
     */
    public static int getDaysBetween(String smallDate, String bigDate) {
        // 日期格式
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        // 获取两个日期的时间戳
        Calendar cal = Calendar.getInstance();
        try {
            cal.setTime(sdf.parse(smallDate));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        long smallTime = cal.getTimeInMillis();
        try {
            cal.setTime(sdf.parse(bigDate));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        long bigTime = cal.getTimeInMillis();
        // 相差的日期
        long days = (bigTime - smallTime) / (1000 * 3600 * 24);
        // long转int 存在溢出情况  根据业务情况编辑catch中返回值
        try {
            return Integer.parseInt(String.valueOf(days));
        } catch (NumberFormatException e) {
            e.printStackTrace();
            return 0;
        }
    }


}
